"""
The `Frames
<https://docs.rs/pauli_tracker/latest/pauli_tracker/tracker/frames/struct.Frames.html>`_ Pauli tracker.

This tracker when building up a circuit. It trackes all Pauli frames in a big matrix.
"""

from pauli_tracker._lib.frames import PartialOrderGraph, StackedTransposed
