"""
`Frames
<https://docs.rs/pauli_tracker/latest/pauli_tracker/tracker/frames/struct.Frames.html>`_\\
<`Map
<https://docs.rs/pauli_tracker/latest/pauli_tracker/collection/type.Map.html>`_>
"""

from pauli_tracker._lib.frames.map import Frames
