macro_rules! live {
    () => {
        "https://docs.rs/pauli_tracker/latest/pauli_tracker/tracker/live/struct.Live.html"
    };
}
pub(crate) use live;

macro_rules! frames {
    () =>
        {
"https://docs.rs/pauli_tracker/latest/pauli_tracker/tracker/frames/struct.Frames.html"
        }
}
pub(crate) use frames;

macro_rules! map {
    () => {
        "https://docs.rs/pauli_tracker/latest/pauli_tracker/collection/type.Map.html"
    };
}
pub(crate) use map;

macro_rules! naive_vector {
    () =>
        {
"https://docs.rs/pauli_tracker/latest/pauli_tracker/collection/struct.NaiveVector.html"
        }
}
pub(crate) use naive_vector;

macro_rules! pauli_dense {
    () => {
        "https://docs.rs/pauli_tracker/latest/pauli_tracker/pauli/struct.PauliDense.html"
    };
}
pub(crate) use pauli_dense;

macro_rules! pauli_stack {
    () => {
        "https://docs.rs/pauli_tracker/latest/pauli_tracker/pauli/struct.PauliStack.html"
    };
}
pub(crate) use pauli_stack;

macro_rules! bit_vec {
    () => {
        "https://docs.rs/bitvec/latest/bitvec/vec/struct.BitVec.html"
    };
}
pub(crate) use bit_vec;
