"""
The `Live
<https://docs.rs/pauli_tracker/latest/pauli_tracker/tracker/live/struct.Live.html>`_ Pauli tracker.

This tracker can be used during execution of the circuit to dynamically update the
Paulis, i.e., 'adding them up'.
"""
